import stan
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import arviz as az
import pickle

data_frame = pd.read_excel('table_2groups.xlsx')
print(data_frame)

col_names = data_frame.keys()[1:]
row_names = data_frame['年齢'].values
print('row_names =\n', row_names)
print('col_names =\n', col_names)
x = data_frame.values[:,1:]
print(x)

stan_code = """
    data {
        int n_col;
        int n_row;
        array[n_row, n_col] int x;
        array[n_row] real n_pop;
    }
    parameters {
        real ab0;
        array[n_col-1] real b;
        array[n_row-1] real a;
        array[n_row-1, n_col-1] real c;
    }
    transformed parameters {
        array[n_row, n_col] real lmbd;
        for (i in 1:n_row)
            for (j in 1:n_col) {
                if (i == 1) {
                    if (j == 1) {
                        lmbd[i][j] = exp(ab0);
                    }
                    else {
                        lmbd[i][j] = exp(ab0 + b[j-1]); 
                    }
                }
                else {
                    if (j == 1) {
                        lmbd[i][j] = exp(ab0 + a[i-1]);
                    }
                    else {
                        lmbd[i][j] = exp(ab0 + a[i-1] + b[j-1] + c[i-1][j-1]);
                    }
                }
            }

    }
    model {
        ab0 ~ normal(0, 10);
        a ~ normal(0, 10);
        b ~ normal(0, 10);
        for (i in 1:n_row-1)
            for (j in 1:n_col-1)
                c[i][j] ~ normal(0, 10);
        for (i in 1:n_row)
            for (j in 1:n_col) {
                x[i][j] ~ poisson(n_pop[i]*lmbd[i][j]);
            }
    }
"""

n_pops = [10781259, 2479294]

Data = {'n_row':len(row_names), 'n_col':len(col_names), 'x':x, 'n_pop':n_pops}
sm = stan.build(stan_code, data=Data)
fit = sm.sample()
print(fit)
i_data = az.from_pystan(posterior=fit, posterior_model=sm)
smmry = az.summary(i_data)

pd.set_option('display.max_rows', None)
print(smmry)

fit_df = fit.to_frame()
print(fit)

with open('fit.pkl', 'wb') as f:
    pickle.dump(fit_df, f)
